#include <stdlib.h>
#include <string.h>
#include "dstring.h"

#ifndef min
#define min(a, b)                          (((a) <= (b)) ? (a) : (b))
#endif

DSTRING::DSTRING()
{
	String = (char *)malloc(16);
	Size = 16;
	String[0] = '\0';
}

DSTRING::DSTRING(const DSTRING &str)
{
	Size = str.Size;
	String = (char *)malloc(Size);
	strcpy(String, str.String);
}

DSTRING::DSTRING(int size)
{
	if(size < 2)
		size = 2;
	String = (char *)malloc(size);
	Size = size;
	String[0] = '\0';
}

DSTRING::DSTRING(const char *str)
{
	Size = strlen(str) + 1;
	String = (char *)malloc(Size);
	strcpy(String, str);
}

DSTRING::~DSTRING()
{
	free(String);
}

int DSTRING::Length(void) const
{
	return strlen(String);
}

bool DSTRING::Resize(int size)
{
	int c;
	char *temp;

	if(size < 2)
		return false;

	temp = (char *)malloc(size);
	for(c = 0; c < min(signed(strlen(String)), size - 1); c++)
		temp[c] = String[c];
	temp[c] = '\0';
	free(String);
	Size = size;
	String = temp;

	return true;
}

DSTRING DSTRING::Substr(int start, int end) const
{
	int c, p;
	DSTRING temp;

	if(start < 0)
		start = 0;
	if(end >= Length())
		end = Length() - 1;

	if(start >= end)
		return temp;

	if(end - start >= 16)
		temp.Resize(end - start + 1);

	p = 0;
	for(c = start; c <= end; c++)
	{
		temp[p] = String[c];
		p++;
	}
	temp[p] = '\0';

	return temp;
}

char &DSTRING::operator [] (int index)
{
	return String[index];
}

char DSTRING::operator [] (int index) const
{
	return String[index];
}

const DSTRING &DSTRING::operator = (const DSTRING &rhs)
{
	if(rhs.Length() >= Size)
	{
		free(String);
		Size = rhs.Length() + 1;
		String = (char *)malloc(Size);
	}
	strcpy(String, rhs.String);

	return *this;
}

const DSTRING &DSTRING::operator = (const char *rhs)
{
	if(signed(strlen(rhs)) >= Size)
	{
		free(String);
		Size = strlen(rhs) + 1;
		String = (char *)malloc(Size);
	}
	strcpy(String, rhs);

	return *this;
}

const DSTRING &DSTRING::operator = (int rhs)
{
	char buf[16];

	itoa(rhs, buf, 10);
	if(signed(strlen(buf)) >= Size)
	{
		free(String);
		Size = strlen(buf) + 1;
		String = (char *)malloc(Size);
	}
	strcpy(String, buf);

	return *this;
}

DSTRING::operator int() const
{
	return atoi(String);
}

DSTRING::operator char *()
{
	return String;
}

DSTRING::operator const char *() const
{
	return String;
}

const DSTRING &DSTRING::operator += (const DSTRING &rhs)
{
	if(rhs.Length() + Length() >= Size)
		Resize(rhs.Length() + Length() + 1);

	strcat(String, rhs.String);
	
	return *this;
}

const DSTRING &DSTRING::operator += (const char *rhs)
{
	if(signed(strlen(rhs)) + Length() >= Size)
		Resize(strlen(rhs) + Length() + 1);

	strcat(String, rhs);

	return *this;
}

const DSTRING &DSTRING::operator += (char rhs)
{
	int p = Length();

	if(p + 1 >= Size)
		Resize(p + 2);

	String[p] = rhs;
	String[p + 1] = '\0';

	return *this;
}

const DSTRING &DSTRING::operator += (int rhs)
{
	char buf[16];

	itoa(rhs, buf, 10);
	
	return (operator += (buf));
}

DSTRING operator + (const DSTRING &lhs, const DSTRING &rhs)
{
	DSTRING temp(lhs.Length() + rhs.Length() + 1);

	temp = lhs;
	temp += rhs;

	return temp;
}

DSTRING operator + (const DSTRING &lhs, const char *rhs)
{
	DSTRING temp(lhs.Length() + strlen(rhs) + 1);

	temp = lhs;
	temp += rhs;

	return temp;
}

DSTRING operator + (const char *lhs, const DSTRING &rhs)
{
	DSTRING temp(strlen(lhs) + rhs.Length() + 1);

	temp = lhs;
	temp += rhs;

	return temp;
}

DSTRING operator + (const DSTRING &lhs, char rhs)
{
	DSTRING temp(lhs.Length() + 2);

	temp = lhs;
	temp += rhs;
	
	return temp;
}

DSTRING operator + (char lhs, const DSTRING &rhs)
{
	DSTRING temp(rhs.Length() + 2);

	temp += lhs;
	temp += rhs;

	return temp;
}

bool operator == (const DSTRING &lhs, const DSTRING &rhs)
{
	if(!strcmp(lhs, rhs))
		return true;
	return false;
}

bool operator != (const DSTRING &lhs, const DSTRING &rhs)
{
	if(!strcmp(lhs, rhs))
		return false;
	return true;
}

bool operator == (const DSTRING &lhs, const char *rhs)
{
	if(!strcmp(lhs, rhs))
		return true;
	return false;
}

bool operator == (const char *lhs, const DSTRING &rhs)
{
	if(!strcmp(lhs, rhs))
		return true;
	return false;
}

bool operator != (const DSTRING &lhs, const char *rhs)
{
	if(!strcmp(lhs, rhs))
		return false;
	return true;
}

bool operator != (const char *lhs, const DSTRING &rhs)
{
	if(!strcmp(lhs, rhs))
		return false;
	return true;
}
