#include <windows.h>
#include <stdio.h>
#include <string.h>
#include "modlist.h"

extern char DFPath[];

void check_path(char *path)
{
	int l = strlen(path);
	if(path[l - 1] != '\\')
	{
		path[l] = '\\';
		path[l + 1] = '\0';
	}
}

static bool ispath(int c)
{
	if(c >= ' ' && c < '~')
	{
		if(c != '\"' && c != ',' && c != '/' && c != '|')
			return true;
	}
	return false;
}

void extract_path(const char *data, char *buf)
{
	int l = 4;
	
	while(ispath(*data))
	{
		data--;
		l++;
	}

	data++;
	l--;
	strncpy(buf, data, l);
	buf[l] = '\0';
}

int file_exists(const char *fname)
{
	static WIN32_FILE_ATTRIBUTE_DATA wfad;
	if(!GetFileAttributesEx(fname, GetFileExInfoStandard, &wfad))
		return 0;
	return wfad.nFileSizeLow;
}

int file_exists2(char *fname)
{
	int l = strlen(fname);
	int s;

	strcpy(&fname[l - 3], "dds");
	if((s = file_exists(fname)) != 0)
		return s;

	strcpy(&fname[l - 3], "bmp");
	if((s = file_exists(fname)) != 0)
		return s;

	strcpy(&fname[l - 3], "tga");
	if((s = file_exists(fname)) != 0)
		return s;

	return 0;
}

int file_exists3(char *fname)
{
	int l = strlen(fname);
	int s;

	strcpy(&fname[l - 3], "wav");
	if((s = file_exists(fname)) != 0)
		return s;

	strcpy(&fname[l - 3], "mp3");
	if((s = file_exists(fname)) != 0)
		return s;

	return 0;
}

int find_file2(char *fname)
{
	int l = strlen(fname);
	int s = strlen(DFPath);
	int fs;
	char buf[256];

	switch(tolower(fname[l - 1]))
	{
	case 'a':
	case 'p':
	case 's':
		sprintf(buf, "%sTextures\\%s", DFPath, fname);
		if((fs = file_exists2(buf)) == 0)
			return 0;
		break;

	case 'f':
		sprintf(buf, "%sMeshes\\%s", DFPath, fname);
		if((fs = file_exists(buf)) == 0)
			return 0;
		break;

	default:
		return 0;
	}

	strcpy(fname, &buf[s]);
	return fs;
}

int check_extension(const char *fname)
{
	int l = strlen(fname);

	if(l <= 4)
		return 0;

	if(stricmp(&fname[l - 3], "nif") == 0)
		return EXT_NIF;
	if(stricmp(&fname[l - 3], "bmp") == 0)
		return EXT_BMP;
	if(stricmp(&fname[l - 3], "tga") == 0)
		return EXT_TGA;
	if(stricmp(&fname[l - 3], "dds") == 0)
		return EXT_DDS;
	if(stricmp(&fname[l - 3], "esp") == 0)
		return EXT_ESP;
	if(stricmp(&fname[l - 3], "esm") == 0)
		return EXT_ESM;
	if(stricmp(&fname[l - 3], "wav") == 0)
		return EXT_WAV;
	if(stricmp(&fname[l - 3], "mp3") == 0)
		return EXT_MP3;
	if(stricmp(&fname[l - 2], "kf") == 0) // added some additional file types to recognize in the zip file's we install...
		return EXT_KF;
	if(stricmp(&fname[l - 2], "db") == 0)
		return EXT_DB;
	if(stricmp(&fname[l - 3], "fnt") == 0)
		return EXT_FNT;
	if(stricmp(&fname[l - 3], "tex") == 0)
		return EXT_TEX;
	if(stricmp(&fname[l - 3], "ifl") == 0)
		return EXT_IFL;

	return 0;
}

static const char *stristr(const char *str, const char *search)
{
	int l1 = strlen(str);
	int l2 = strlen(search);

	for(int c = 0; c < l1 - l2; c++)
	{
		if(strnicmp(&str[c], search, l2) == 0)
			return &str[c];
	}

	return NULL;
}

void check_path_sep(char *path)
{
	int l = strlen(path);
	for(int c = 0; c < l; c++)
	{
		if(path[c] == '/')
			path[c] = '\\';
	}
}

bool shorten_path(char *path)
{
	int c;
	int l = strlen(path);
	char buf[256];
	const char *ptr;

	if(stricmp(&path[l - 3], "esp") == 0 || stricmp(&path[l - 3], "esm") == 0)
	{
		for(c = l - 1; c >= 0; c--)
		{
			if(path[c] == '\\')
			{
				strcpy(buf, &path[c + 1]);
				break;
			}
			else if(c == 0)
				strcpy(buf, path);
		}
	}
	else if((ptr = stristr(path, "Meshes\\")) || (ptr = stristr(path, "Textures\\")) ||
			(ptr = stristr(path, "Icons\\")) || (ptr = stristr(path, "BookArt\\")) ||
			(ptr = stristr(path, "Sound\\")) || (ptr = stristr(path, "Music\\"))) // added check for "Music" path
		strcpy(buf, ptr);
	else
		return false;

	strcpy(path, buf);
	return true;
}

int compare_file_date(const char *fname, FILETIME *ft)
{
	WIN32_FILE_ATTRIBUTE_DATA wfad;
	memset(&wfad, 0, sizeof(wfad));
	if(!file_exists(fname))
		return -1;

	GetFileAttributesEx(fname, GetFileExInfoStandard, &wfad);
	if(CompareFileTime(ft, &wfad.ftLastWriteTime) == 0)
		return 0;

	return 1;
}

void gzputdw(gzFile file, int d)
{
	gzputc(file, (unsigned char)d);
	gzputc(file, (unsigned char)(d >> 8));
	gzputc(file, (unsigned char)(d >> 16));
	gzputc(file, (unsigned char)(d >> 24));
}

int gzgetdw(gzFile file)
{
	int d;
	d = gzgetc(file);
	d |= gzgetc(file) << 8;
	d |= gzgetc(file) << 16;
	d |= gzgetc(file) << 24;
	return d;
}

void create_dir(const char *path)
{
	int c, l1, l2;
	char buf[256];

	l1 = strlen(path);
	l2 = strlen(DFPath);
	for(c = l2 + 1; c < l1; c++)
	{
		if(path[c] == '\\')
		{
			strncpy(buf, path, c);
			buf[c] = '\0';
			CreateDirectory(buf, NULL);
		}
	}
}

void remove_dir(const char *path)
{
	int c, l1, l2;
	char buf[256];

	l1 = strlen(path);
	l2 = strlen(DFPath);
	for(c = l1 - 1; c > l2 + 1; c--)
	{
		if(path[c] == '\\')
		{
			strncpy(buf, path, c);
			buf[c] = '\0';
			RemoveDirectory(buf);
		}
	}
}
