#ifndef _LLIST_H_
#define _LLIST_H_

#define LLIST_END               0
#define LLIST_START             1
#define LLIST_POS               2

template <class Type>
struct LISTNODE
{
	Type *Item;
	LISTNODE *Next;
	LISTNODE *Prev;
};

template <class Type>
class LLIST
{
public:
	LLIST();
	~LLIST();

	int Length(void)       { return nItems; }
	Type &GetItem(void)    { return *(Current->Next->Item); }
	
	Type &NewItem(void);
	void NewPtr(Type *item);
	bool DeleteItem(void);
	void DeleteAll(void);

	bool NextItem(void);
	bool PrevItem(void);
	void GotoStart(void);
	void GotoEnd(void); 

	void MoveToStart(void);

	void SetInsertMode(int mode);
	LLIST &operator += (const Type &rhs);
	LLIST &operator = (LLIST &rhs);

private:
	LISTNODE<Type> *Head;
	LISTNODE<Type> *Tail;
	LISTNODE<Type> *Current;
	int nItems;
	int InsertPos;
};

template <class Type>
LLIST<Type>::LLIST()
{
	Head = new LISTNODE<Type>;
	Tail = new LISTNODE<Type>;
	
	Head->Item = NULL;
	Head->Next = Tail;
	Head->Prev = NULL;

	Tail->Item = NULL;
	Tail->Next = NULL;
	Tail->Prev = Head;

	Current = Head;
	nItems = 0;
	InsertPos = LLIST_END;
}

template <class Type>
LLIST<Type>::~LLIST()
{
	InsertPos = LLIST_START;
	while(DeleteItem()) { }
	delete Head;
	delete Tail;
}

template <class Type>
void LLIST<Type>::DeleteAll(void)
{
	int pos = InsertPos;
	InsertPos = LLIST_START;
	while(DeleteItem()) { }
	InsertPos = pos;
}

template <class Type>
Type &LLIST<Type>::NewItem(void)
{
	LISTNODE <Type> *Temp, *NewNode;
	NewNode = new LISTNODE<Type>;
	NewNode->Item = new Type;

	switch(InsertPos)
	{
	case LLIST_END:
		Temp = Tail->Prev;
		Tail->Prev = NewNode;
		Temp->Next = NewNode;
		NewNode->Next = Tail;
		NewNode->Prev = Temp;
		break;
	case LLIST_START:
		Temp = Head->Next;
		Head->Next = NewNode;
		Temp->Prev = NewNode;
		NewNode->Next = Temp;
		NewNode->Prev = Head;
		break;
	case LLIST_POS:
		Temp = Current->Next;
		Current->Next = NewNode;
		Temp->Prev = NewNode;
		NewNode->Next = Temp;
		NewNode->Prev = Current;
		break;
	}

	nItems++;

	return *(NewNode->Item);
}

template <class Type>
void LLIST<Type>::NewPtr(Type *item)
{
	LISTNODE <Type> *Temp, *NewNode;
	NewNode = new LISTNODE<Type>;
	NewNode->Item = item;

	switch(InsertPos)
	{
	case LLIST_END:
		Temp = Tail->Prev;
		Tail->Prev = NewNode;
		Temp->Next = NewNode;
		NewNode->Next = Tail;
		NewNode->Prev = Temp;
		break;
	case LLIST_START:
		Temp = Head->Next;
		Head->Next = NewNode;
		Temp->Prev = NewNode;
		NewNode->Next = Temp;
		NewNode->Prev = Head;
		break;
	case LLIST_POS:
		Temp = Current->Next;
		Current->Next = NewNode;
		Temp->Prev = NewNode;
		NewNode->Next = Temp;
		NewNode->Prev = Current;
		break;
	}
	nItems++;
}

template <class Type>
LLIST<Type> &LLIST<Type>::operator += (const Type &rhs)
{
	NewItem() = rhs;

	return *this;
}

template <class Type>
bool LLIST<Type>::DeleteItem(void)
{
	LISTNODE <Type> *DelNode, *Temp;

	if(nItems <= 0)
		return false;

	switch(InsertPos)
	{
	case LLIST_END:
		DelNode = Tail->Prev;
		Temp = DelNode->Prev;
		delete DelNode->Item;
		delete DelNode;
		Temp->Next = Tail;
		Tail->Prev = Temp;
		break;
	case LLIST_START:
		DelNode = Head->Next;
		Temp = DelNode->Next;
		delete DelNode->Item;
		delete DelNode;
		Temp->Prev = Head;
		Head->Next = Temp;
		break;
	case LLIST_POS:
		DelNode = Current->Next;
		Temp = DelNode->Next;
		delete DelNode->Item;
		delete DelNode;
		Temp->Prev = Current;
		Current->Next = Temp;
		break;
	}

	nItems--;
	
	return true;
}

template <class Type>
bool LLIST<Type>::NextItem(void)
{
	if(Current->Next == Tail)
		return false;
	if(Current->Next->Next == Tail)
		return false;

	Current = Current->Next;
	
	return true;
}

template <class Type>
bool LLIST<Type>::PrevItem(void)
{
	if(Current->Prev == Head)
		return false;

	Current = Current->Prev;

	return true;
}

template <class Type>
void LLIST<Type>::GotoStart(void)
{
	Current = Head;
}

template <class Type>
void LLIST<Type>::GotoEnd(void)
{
	if(nItems <= 0)
		Current = Head;
	else
		Current = Tail->Prev->Prev;
}

template <class Type>
void LLIST<Type>::MoveToStart(void)
{
	LISTNODE <Type> *Temp, *NewNode;
	
	if(Current == Head)
		return;

	NewNode = Current->Next;
	Temp = Head->Next;
	Head->Next = NewNode;
	Temp->Prev = NewNode;
	NewNode->Prev->Next = NewNode->Next;
	NewNode->Next->Prev = NewNode->Prev;
	NewNode->Next = Temp;
	NewNode->Prev = Head;
}

template <class Type>
void LLIST<Type>::SetInsertMode(int mode)
{
	switch(mode)
	{
	case LLIST_END:
	case LLIST_START:
	case LLIST_POS:
		InsertPos = mode;
		break;
	}
}

template <class Type>
LLIST<Type> &LLIST<Type>::operator = (LLIST<Type> &rhs)
{
	DeleteAll();
	LISTNODE <Type> *Temp;
	int pos;

	if(rhs.nItems > 0)
	{
		Temp = rhs.Current;
		pos = rhs.InsertPos;
		rhs.InsertPos = LLIST_END;
		rhs.Current = rhs.Head;

		do{
			*this += rhs.GetItem();
		}while(rhs.NextItem());

		rhs.Current = Temp;
		rhs.InsertPos = pos;
	}

	return *this;
}

#endif
