#include <windows.h>
#include <stdio.h>
#include "dstring.h"
#include "modlist.h"

int file_exists(const char *fname);

MOD::MOD()
{
	ModifyTime.dwLowDateTime = 0;
	ModifyTime.dwHighDateTime = 0;
	FileInfo = NULL;
	nFiles = 0;
	nFilesAlloc = 0;
	Flags = 0;
	Size = 0;
}

MOD::~MOD()
{
	Destroy();
}

void MOD::Destroy(void)
{
	for(int c = 0; c < nFiles; c++)
	{
		if(FileInfo[c].Name)
		{
			free(FileInfo[c].Name);
			FileInfo[c].Name = NULL;
		}
	}
	if(FileInfo)
	{
		free(FileInfo);
		FileInfo = NULL;
	}
	nFiles = 0;
	nFilesAlloc = 0;
}

bool MOD::Save(gzFile zf)
{
	gzputdw(zf, Name.Length());
    gzputs(zf, Name);
	gzwrite(zf, &ModifyTime, sizeof(FILETIME));
	gzputdw(zf, Flags);
	gzputdw(zf, nFiles);
	gzputdw(zf, Size);

	for(int c = 0; c < nFiles; c++)
	{
		gzputdw(zf, strlen(FileInfo[c].Name));
		gzputs(zf, FileInfo[c].Name);
		gzputdw(zf, FileInfo[c].Type);
	}
	return true;
}

bool MOD::Load(gzFile zf)
{
	int c, n;
	Destroy();
	c = gzgetdw(zf);
	Name.Resize(c + 1);
	gzgets(zf, Name, c + 1);
	gzread(zf, &ModifyTime, sizeof(FILETIME));
	Flags = gzgetdw(zf);
	nFiles = gzgetdw(zf);
	Size = gzgetdw(zf);
	nFilesAlloc = nFiles;
	if(nFiles > 0)
		FileInfo = (MODFILE *)malloc(sizeof(MODFILE) * nFilesAlloc);

	for(c = 0; c < nFiles; c++)
	{
		n = gzgetdw(zf);
		FileInfo[c].Name = (char *)malloc(n + 1);
		gzgets(zf, FileInfo[c].Name, n + 1);
		FileInfo[c].Type = gzgetdw(zf);
	}
	return true;
}

bool MOD::AddFile(const char *fname, DWORD type)
{
	int c;
	for(c = 0; c < nFiles; c++)
	{
		if(stricmp(FileInfo[c].Name, fname) == 0)
			return true;
	}

	if(nFiles >= nFilesAlloc)
	{
		nFilesAlloc = nFiles + 32;
		FileInfo = (MODFILE *)realloc(FileInfo, sizeof(MODFILE) * nFilesAlloc);
	}

	c = strlen(fname) + 1;
	FileInfo[nFiles].Name = (char *)malloc(c);
	strcpy(FileInfo[nFiles].Name, fname);
	FileInfo[nFiles].Type = type;
	nFiles++;
	return true;
}

bool MOD::RemoveFile(const char *fname)
{
	for(int c = 0; c < nFiles; c++)
	{
		if(stricmp(FileInfo[c].Name, fname) == 0)
		{
			free(FileInfo[c].Name);
			nFiles--;
			if(c < nFiles)
				memcpy(&FileInfo[c], &FileInfo[nFiles], sizeof(MODFILE));
			return true;
		}
	}
	return false;
}


void MOD::RemoveAuto(void)
{
	for(int c = 0; c < nFiles; c++)
	{
		if(!(FileInfo[c].Type & MF_MANUAL))
		{
			free(FileInfo[c].Name);
			nFiles--;
			if(c < nFiles)
				memcpy(&FileInfo[c], &FileInfo[nFiles], sizeof(MODFILE));
			c--;
		}
	}
}

void MOD::Merge(MOD *mod)
{
	for(int c = 0; c < mod->nFiles; c++)
		AddFile(mod->FileInfo[c].Name, mod->FileInfo[c].Type);
}
