#include <windows.h>
#include <commdlg.h>
#include <string.h>
#include "ofn.h"

OFNWINDOW::OFNWINDOW()
{
	InitialPath = ".";
	memset(&Ofn, 0, sizeof(Ofn));
	Ofn.lStructSize = sizeof(Ofn);
	Ofn.nMaxFile = 256;
	Ofn.lpstrFile = CurrentPath;
	CurrentPath[0] = '\0';
}

void OFNWINDOW::UpdateFileInfo(void)
{
	int c;

	SelFiles = 0;

	if(CurrentPath[Ofn.nFileOffset - 1] != '\0')
	{
		SelFiles = 1;
		CurrentFile = 0;
		CurrentFileName = CurrentPath;
		return;
	}

	for(c = Ofn.nFileOffset; c < 255; c++)
	{
		if(CurrentPath[c] == '\0')
		{
			SelFiles++;
			if(CurrentPath[c + 1] == '\0')
				break;
		}
	}

	CurrentFile = -1;
	NextFile();
}

bool OFNWINDOW::Display(HINSTANCE hInstance, HWND hwnd, const char *title, DWORD flags, int type)
{
	bool res;
	int error;
	Ofn.hInstance = hInstance;
	Ofn.hwndOwner = hwnd;
	Ofn.lpstrTitle = title;
	Ofn.Flags = flags;

	switch(type)
	{
	case OFNWINDOW_OPEN:
		res = (GetOpenFileName(&Ofn) != 0) ? true : false;
		error = CommDlgExtendedError();
		if(res)
			UpdateFileInfo();
		return res;
	case OFNWINDOW_SAVE:
		res = (GetSaveFileName(&Ofn) != 0) ? true : false;
		if(res)
			UpdateFileInfo();
		return res;
	}

	return false;
}

void OFNWINDOW::SetInitialPath(const char *path)
{
	InitialPath = path;
	Ofn.lpstrInitialDir = InitialPath;
}

void OFNWINDOW::SetDefaultExt(const char *ext)
{
	strcpy(DefaultExt, ext);
	Ofn.lpstrDefExt = DefaultExt;
}

void OFNWINDOW::SetFilter(const char *filter)
{
	int c;

	Filter = filter;
	Filter += ' ';

	for(c = 0; c <= (signed)strlen(filter); c++)
	{
		if(filter[c] == ',' || filter[c] == '\0')
			Filter[c] = '\0';
	}

	Filter[(int)(strlen(filter) + 1)] = '\0';
	Ofn.lpstrFilter = Filter;
}

char *OFNWINDOW::GetFullPath(void)
{
	return CurrentFileName;
}

char *OFNWINDOW::GetFileName(void)
{
	int c;

	for(c = CurrentFileName.Length() - 1; c >= 0; c--)
	{
		if(CurrentFileName[c] == '\\')
			return &CurrentFileName[c + 1];
	}
	
	return CurrentFileName;
}

char *OFNWINDOW::GetExtension(void)
{
	int c;

	for(c = CurrentFileName.Length() - 1; c >= 0; c--)
	{
		if(CurrentFileName[c] == '.')
			return &CurrentFileName[c + 1];
	}

	return &CurrentFileName[CurrentFileName.Length()];
}

bool OFNWINDOW::NextFile(void)
{
	int c, n = 0;

	if(CurrentFile >= SelFiles - 1)
		return false;

	CurrentFile++;
	CurrentFileName = CurrentPath;
	CurrentFileName += '\\';

	for(c = Ofn.nFileOffset; c < 256; c++)
	{
		if(n == CurrentFile)
			break;

		if(CurrentPath[c] == '\0')
			n++;
	}

	CurrentFileName += &CurrentPath[c];
	return true;
}
