#include <windows.h>
#include <shlwapi.h>
#include <shlobj.h>
#include <zlib.h>
#include <unzip.h>
#include <zip.h>
#include <string.h>
#include <stdlib.h>
#include <commctrl.h>
#include <process.h>
#include "modlist.h"
#include "ofn.h"
#include "resource.h"

struct SCANDATA
{
	char *data;
	int size;
	MOD *mod;
};

struct ARCHIVEDATA
{
	const char *fname;
	MOD *mod;
};

struct CLEANDATA
{
	int scan_sound;
	MOD *mod;
};

extern MODLIST ModList;
extern HWND hMain;
extern HINSTANCE ghInstance;
extern char LocalPath[256];
extern char MWPath[256];
extern char DFPath[256];
extern char ZipPath[256];

static volatile int progress_pos;
static volatile int progress_state;

BOOL CALLBACK OverwriteDlgProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);

static BOOL CALLBACK ProgressDlgProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	switch(msg)
	{
	case WM_INITDIALOG:
		SetWindowText(hwnd, (LPCTSTR)lparam);
		SetTimer(hwnd, 1, 250, NULL);
		return TRUE;

	case WM_TIMER:
		SendDlgItemMessage(hwnd, IDC_PROGRESS1, PBM_SETPOS, progress_pos, 0);
		if(progress_state != 0)
		{
			KillTimer(hwnd, 1);
			EndDialog(hwnd, 1);
		}
		return TRUE;

	case WM_DESTROY:
		return TRUE;
	}

	return FALSE;
}

static void scan_nif(const char *fname, MOD *mod)
{
	int fs;
	int size;
	char *data;
	bool gotpath;
	char buf[256];
	FILE *f = fopen(fname, "rb");
	fseek(f, 0, SEEK_END);
	size = ftell(f);
	data = (char *)malloc(size + 1);
	fseek(f, 0, SEEK_SET);
	fread(data, size, 1, f);
	fclose(f);

	gotpath = false;
	for(int c = 0; c < size - 4; c++)
	{
		if(strnicmp(&data[c], ".tga", 4) == 0)
			gotpath = true;
		else if(strnicmp(&data[c], ".bmp", 4) == 0)
			gotpath = true;
		else if(strnicmp(&data[c], ".dds", 4) == 0)
			gotpath = true;
		else if(strnicmp(&data[c], ".kf", 3) == 0)
			gotpath = true;

		if(gotpath)
		{
			extract_path(&data[c], buf);
			fs = find_file2(buf);
			if(fs > 0)
			{
				mod->AddFile(buf, 0);
				mod->Size += fs;
			}
			gotpath = false;
		}
	}
	free(data);
}

static int find_file(char *fname, MOD *mod)
{
	int l = strlen(fname);
	int s = strlen(DFPath);
	int fs;
	char buf[256];

	switch(tolower(fname[l - 1]))
	{
	case 'f':
		sprintf(buf, "%sMeshes\\%s", DFPath, fname);
		if((fs = file_exists(buf)) == 0)
			return 0;
		scan_nif(buf, mod);
		break;

	case 'a':
	case 'p':
	case 's':
		sprintf(buf, "%sBookArt\\%s", DFPath, fname);
		if((fs = file_exists2(buf)) == 0)
		{
			sprintf(buf, "%sIcons\\%s", DFPath, fname);
			if((fs = file_exists2(buf)) == 0)
				return 0;
		}
		break;

	case 'v':
	case '3':
		sprintf(buf, "%sSound\\%s", DFPath, fname);
		if((fs = file_exists3(buf)) == 0)
			return 0;
		break;

	default:
		return 0;
	}

	strcpy(fname, &buf[s]);
	return fs;
}

static void scan_thread(void *ptr)
{
	SCANDATA *sd = (SCANDATA *)ptr;
	bool gotpath = false;
	char buf[256];
	int fs;

	for(int c = 0; c < sd->size - 4; c++)
	{
		progress_pos = (int)(((float)c / (float)(sd->size - 4)) * 100.0f);

		if(strnicmp(&sd->data[c], ".nif", 4) == 0)
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".tga", 4) == 0)
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".bmp", 4) == 0)
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".dds", 4) == 0)
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".wav", 4) == 0)
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".mp3", 4) == 0)
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".fnt", 4) == 0) // added additional file types to check for... but this doesn't seem to work. for instance .kf files don't seem to be referenced in the .esp's
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".tex", 4) == 0)
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".ifl", 4) == 0)
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".kf", 3) == 0)
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".db", 3) == 0)
			gotpath = true;


		if(gotpath)
		{
			extract_path(&sd->data[c], buf);
			fs = find_file(buf, sd->mod);
			if(fs > 0)
			{
				sd->mod->AddFile(buf, 0);
				sd->mod->Size += fs;
			}
			gotpath = false;
		}
	}
	progress_state = 1;
	_endthread();
}

MOD *scan_esp(const char *fname)
{
	SCANDATA sd;
	char buf[256];
	WIN32_FILE_ATTRIBUTE_DATA wfad;
	memset(&wfad, 0, sizeof(wfad));
	GetFileAttributesEx(fname, GetFileExInfoStandard, &wfad);
	FILE *f = fopen(fname, "rb");
	if(!f)
		return false;
	fseek(f, 0, SEEK_END);
	sd.size = ftell(f);
	sd.data = (char *)malloc(sd.size + 1);
	fseek(f, 0, SEEK_SET);
	fread(sd.data, sd.size, 1, f);
	fclose(f);
	sd.mod = new MOD;

	strcpy(buf, fname);
	memcpy(&sd.mod->ModifyTime, &wfad.ftLastWriteTime, sizeof(FILETIME));
	shorten_path(buf);
	sd.mod->Name = buf;
	sd.mod->Size = sd.size;

	sd.mod->AddFile(buf, 0);
	sprintf(buf, "Scanning %s", (const char *)sd.mod->Name);

	progress_pos = 0;
	progress_state = 0;
	_beginthread(&scan_thread, 0, &sd);
	DialogBoxParam(ghInstance, MAKEINTRESOURCE(IDD_PROGRESSDIALOG), hMain, ProgressDlgProc, (LPARAM)buf);
	free(sd.data);
	return sd.mod;
}

static void scan_esm_thread(void *ptr)
{
	SCANDATA *sd = (SCANDATA *)ptr;
	bool gotpath = false;
	char buf[256];
	int fs;

	for(int c = 0; c < sd->size - 4; c++)
	{
		progress_pos = (int)(((float)c / (float)(sd->size - 4)) * 100.0f);

		if(strnicmp(&sd->data[c], ".wav", 4) == 0)
			gotpath = true;
		else if(strnicmp(&sd->data[c], ".mp3", 4) == 0)
			gotpath = true;

		if(gotpath)
		{
			extract_path(&sd->data[c], buf);
			if(strnicmp(buf, "Cr", 2) == 0 || strnicmp(buf, "Fx", 2) == 0 || strnicmp(buf, "Vo", 2) == 0)
			{
				fs = find_file(buf, sd->mod);
				if(fs > 0)
				{
					sd->mod->AddFile(buf, 0);
					sd->mod->Size += fs;
				}
			}
			gotpath = false;
		}
	}
	progress_state = 1;
	_endthread();
}

MOD *scan_esm(const char *fname)
{
	SCANDATA sd;
	char buf[256];
	WIN32_FILE_ATTRIBUTE_DATA wfad;
	memset(&wfad, 0, sizeof(wfad));
	GetFileAttributesEx(fname, GetFileExInfoStandard, &wfad);
	FILE *f = fopen(fname, "rb");
	if(!f)
		return false;
	fseek(f, 0, SEEK_END);
	sd.size = ftell(f);
	sd.data = (char *)malloc(sd.size + 1);
	fseek(f, 0, SEEK_SET);
	fread(sd.data, sd.size, 1, f);
	fclose(f);
	sd.mod = new MOD;

	sd.mod->Size = sd.size;
	memcpy(&sd.mod->ModifyTime, &wfad.ftLastWriteTime, sizeof(FILETIME));
	strcpy(buf, fname);
	shorten_path(buf);
	sd.mod->Name = buf;

	sd.mod->AddFile(buf, 0);
	sprintf(buf, "Scanning %s", (const char *)sd.mod->Name);

	progress_pos = 0;
	progress_state = 0;
	_beginthread(&scan_esm_thread, 0, &sd);
	DialogBoxParam(ghInstance, MAKEINTRESOURCE(IDD_PROGRESSDIALOG), hMain, ProgressDlgProc, (LPARAM)buf);
	free(sd.data);
	return sd.mod;
}

static bool install_file(const char *fname, const void *data, int data_len)
{
	int l;
	char buf[256];
	bool same;
	FILE *f;

	sprintf(buf, "%s%s", DFPath, fname);
    
	if(l = file_exists(buf))
	{
		f = fopen(buf, "rb");

		if(l == data_len)
		{
			void *data2 = malloc(l);
			fread(data2, l, 1, f);
			if(memcmp(data, data2, data_len) == 0)
				same = true;
			else
				same = false;
			free(data2);
		}
		else
			same = false;
		
		fclose(f);
		if(same)
			return true;
		else
		{
			if(DialogBoxParam(ghInstance, MAKEINTRESOURCE(IDD_OVERWRITEDIALOG), hMain, OverwriteDlgProc, (LPARAM)fname) == 0)
				return true;
		}
	}

	create_dir(buf);
	f = fopen(buf, "wb");
	fwrite(data, 1, data_len, f);
	fclose(f);
	return true;
}

static void install_thread(void *ptr)
{
	void *data;
	char buf[256];
	int esps = 0;
	int i;
	int res;
	MOD *mod;
	unz_file_info ufi;
	unzFile zf;
	float currentsize = 0.0f;
	float totalsize = 0.0f;

	mod = new MOD;
	zf = unzOpen((const char *)ptr);
	if(!zf)
	{
		delete mod;
		progress_state = -1;
		_endthread();
	}

	unzGoToFirstFile(zf);
	do{
		unzGetCurrentFileInfo(zf, &ufi, buf, 256, NULL, 0, NULL, 0);
		totalsize += (float)ufi.uncompressed_size;
	}while(unzGoToNextFile(zf) == UNZ_OK);

	unzGoToFirstFile(zf);
	do{
		unzGetCurrentFileInfo(zf, &ufi, buf, 256, NULL, 0, NULL, 0);
		if(res = check_extension(buf))
		{
			check_path_sep(buf);
			if(!shorten_path(buf))
			{
				unzClose(zf);
				delete mod;
				progress_state = -1;
				_endthread();
			}

			if((res == EXT_ESM || res == EXT_ESP) && mod->Name[0] == '\0') {  // bug fix: added paren's and changed to == '\0' instead of !=
				mod->Name = buf;
			}
			
			if(mod->AddFile(buf, 0))
			{
				currentsize += (float)ufi.uncompressed_size;
				progress_pos = (int)(currentsize / totalsize * 100.0f);
				data = malloc(ufi.uncompressed_size);
				unzOpenCurrentFile(zf);
				unzReadCurrentFile(zf, data, ufi.uncompressed_size);
				unzCloseCurrentFile(zf);
				if(install_file(buf, data, ufi.uncompressed_size))
					mod->Size += ufi.uncompressed_size;
				free(data);
			}
		}
	}while(unzGoToNextFile(zf) == UNZ_OK);
	unzClose(zf);
	if (ModList.SelectMod(mod->Name)) { // if this mod was already installed, we just re-installed it, so re-save the list of files from the zip file and don't add a second reference to the mod
		ModList.cMod->RemoveAuto(); // remove previous list of files that were automatically added
		for (i=0; i < mod->nFiles; ++i) {
			ModList.cMod->AddFile(mod->FileInfo[i].Name, mod->FileInfo[i].Type); // add the files found in the zipfile to the list
		}
	} else {		
		ModList.ModInfo.NewPtr(mod); // this is a new mod
	}
	ModList.SelectMod(mod->Name);
	progress_state = 1;
	_endthread();
}

bool install_mod(const char *fname)
{
	char buf[256];
	sprintf(buf, "Installing %s", fname);
	progress_pos = 0;
	progress_state = 0;
	_beginthread(&install_thread, 0, (void *)fname);
	DialogBoxParam(ghInstance, MAKEINTRESOURCE(IDD_PROGRESSDIALOG), hMain, ProgressDlgProc, (LPARAM)buf);
	if(progress_state == -1)
	{
		MessageBox(hMain, "Tesmu is unable to install this mod.  Please install it manually.", "Error", MB_OK);
		return false;
	}
	return true;
}

static void archive_thread(void *ptr)
{
	char buf[256];
	void *data;
	int l;
	FILE *f;
	zipFile zf;
	ARCHIVEDATA *ad = (ARCHIVEDATA *)ptr;
	zf = zipOpen(ad->fname, FALSE);

	for(int c = 0; c < ad->mod->nFiles; c++)
	{
		progress_pos = (int)(((float)c / (float)ad->mod->nFiles) * 100.0f);
		sprintf(buf, "%s%s", DFPath, ad->mod->FileInfo[c].Name);
		
		f = fopen(buf, "rb");
		fseek(f, 0, SEEK_END);
		l = ftell(f);
		fseek(f, 0, SEEK_SET);
		data = malloc(l);
		fread(data, 1, l, f);
		fclose(f);

		zipOpenNewFileInZip(zf, ad->mod->FileInfo[c].Name, NULL, NULL, 0, NULL, 0, NULL, Z_DEFLATED, Z_DEFAULT_COMPRESSION);
		zipWriteInFileInZip(zf, data, l);
		zipCloseFileInZip(zf);
		free(data);
	}
	
	zipClose(zf, NULL);
	progress_state = 1;
	_endthread();
}

bool archive_mod(const char *fname, MOD *mod)
{
	char buf[256];
	ARCHIVEDATA ad;
	sprintf(buf, "Archiving %s", fname);
	ad.fname = fname;
	ad.mod = mod;
	progress_pos = 0;
	progress_state = 0;
	_beginthread(&archive_thread, 0, (void *)&ad);
	DialogBoxParam(ghInstance, MAKEINTRESOURCE(IDD_PROGRESSDIALOG), hMain, ProgressDlgProc, (LPARAM)buf);
	return true;
}
