// TESMU2

// Thanks to Rob R. for scanning/file detection fixes

#include <windows.h>
#include <shlwapi.h>
#include <shlobj.h>
#include <zlib.h>
#include <unzip.h>
#include <zip.h>
#include <string.h>
#include <stdlib.h>
#include <commctrl.h>
#include "modlist.h"
#include "ofn.h"
#include "resource.h"

const DWORD TESMU_ID = ('T' | ('S' << 8) | ('M' << 16) | ('U' << 24));
const DWORD TESMU_VER = 0x00020000;

struct TESHEADER
{
	DWORD id;
	DWORD version;
	BYTE remprompt;
	BYTE sortby;
	BYTE sortorder;
	BYTE dateformat;
	BYTE reserved[124];
};

MODLIST ModList;
OFNWINDOW InstallOfn;
OFNWINDOW ScanOfn;
OFNWINDOW AddOfn;
TESHEADER hdr;
HWND hMain;
HINSTANCE ghInstance;
char LocalPath[256];
char MWPath[256];
char DFPath[256];
char ZipPath[256];
char BackupPath[256];
char DocPath[256];

bool get_mw_reg_path(void)
{
	HKEY localkey, bsoftkey, mwkey;
	unsigned long size = 256;

	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE", 0, KEY_READ, &localkey) != ERROR_SUCCESS)
		return false;
	if(RegOpenKeyEx(localkey, "Bethesda Softworks", 0, KEY_READ, &bsoftkey) != ERROR_SUCCESS)
	{
		RegCloseKey(localkey);
		return false;
	}
	if(RegOpenKeyEx(bsoftkey, "Morrowind", 0, KEY_READ, &mwkey) != ERROR_SUCCESS)
	{
		RegCloseKey(bsoftkey);
		RegCloseKey(localkey);
		return false;
	}
	if(RegQueryValueEx(mwkey, "Installed Path", NULL, NULL, (unsigned char *)MWPath, &size) != ERROR_SUCCESS)
	{
		RegCloseKey(mwkey);
		RegCloseKey(bsoftkey);
		RegCloseKey(localkey);
		return false;
	}
	RegCloseKey(mwkey);
	RegCloseKey(bsoftkey);
	RegCloseKey(localkey);
	check_path(MWPath);
	return true;
}

bool get_mw_browse_path(HWND hwnd)
{
	int c, l;
	OFNWINDOW browse;

	browse.SetInitialPath(".");
	browse.SetFilter("Morrowind.exe,morrowind.exe");
	if(!browse.Display(ghInstance, hwnd, "Locate Morrowind.exe...", OFN_HIDEREADONLY | OFN_FILEMUSTEXIST, OFNWINDOW_OPEN))
		return false;

	strcpy(MWPath, browse.GetFullPath());
	l = strlen(MWPath);
	for(c = l - 1; c >= 0; c--)
	{
		if(MWPath[c] == '\\')
		{
			MWPath[c + 1] = '\0';
			return true;
		}
	}

	return false;
}

void save_stuff(void)
{
	char buf[256];
	gzFile zf;

	sprintf(buf, "%sTesmu2.cfg", LocalPath);
	zf = gzopen(buf, "wb");
	gzwrite(zf, &hdr, sizeof(hdr));
	gzputdw(zf, strlen(MWPath));
	gzputs(zf, MWPath);
	gzputdw(zf, strlen(ZipPath));
	gzputs(zf, ZipPath);
	ModList.Save(zf);
	gzclose(zf);
}

int load_stuff(HWND hwnd)
{
	char buf[256];
	int ver;
	int n;
	gzFile zf;

	sprintf(buf, "%sTesmu2.cfg", LocalPath);
	zf = gzopen(buf, "rb");
	if(zf)
	{
		gzread(zf, &hdr, sizeof(hdr));
		if(hdr.id != TESMU_ID)
		{
			gzclose(zf);
			hdr.id = TESMU_ID;
			hdr.version = TESMU_VER;
			hdr.remprompt = TRUE;
			return 0;
		}
		ver = hdr.version;
		if(hdr.version < 0x00020000)
		{
			gzclose(zf);
			hdr.id = TESMU_ID;
			hdr.version = TESMU_VER;
			hdr.remprompt = TRUE;
			return 0;
		}
		n = gzgetdw(zf);
		if(n > 0)
			gzgets(zf, MWPath, n + 1);
		n = gzgetdw(zf);
		if(n > 0)
			gzgets(zf, ZipPath, n + 1);
		ModList.Load(zf);
		gzclose(zf);
		return ver;
	}
	hdr.id = TESMU_ID;
	hdr.version = TESMU_VER;
	hdr.remprompt = TRUE;
	return 0;
}

void update_stuff(HWND hwnd, int offset = 0)
{
	LVITEM lvi;
	int index = ListView_GetSelectionMark(GetDlgItem(hwnd, IDC_MODLIST));
	index += offset;
	if(index < 0)
		index++;
	else if(index >= ModList.ModInfo.Length())
		index--;
	if(index >= 0 && index < ModList.ModInfo.Length())
	{
		memset(&lvi, 0, sizeof(LVITEM));
		lvi.mask = LVIF_PARAM;
		lvi.iItem = index;
		SendDlgItemMessage(hwnd, IDC_MODLIST, LVM_GETITEM, 0, (LPARAM)&lvi);
		ModList.cMod = (MOD *)lvi.lParam;

		EnableWindow(GetDlgItem(hwnd, IDC_INFOBUTTON), TRUE);
		EnableMenuItem(GetMenu(hwnd), ID_MOD_INFO, MF_ENABLED);
		if(stricmp(ModList.cMod->Name, "Morrowind.esm") == 0 || stricmp(ModList.cMod->Name, "Tribunal.esm") == 0 || stricmp(ModList.cMod->Name, "Bloodmoon.esm") == 0)
		{
			EnableWindow(GetDlgItem(hwnd, IDC_REMOVEBUTTON), FALSE);
			EnableWindow(GetDlgItem(hwnd, IDC_ARCHIVEBUTTON), FALSE);
			EnableMenuItem(GetMenu(hwnd), ID_MOD_REMOVE, MF_DISABLED);
			EnableMenuItem(GetMenu(hwnd), ID_MOD_ARCHIVE, MF_DISABLED);
		}
		else
		{
			EnableWindow(GetDlgItem(hwnd, IDC_REMOVEBUTTON), TRUE);
			EnableWindow(GetDlgItem(hwnd, IDC_ARCHIVEBUTTON), TRUE);
			EnableMenuItem(GetMenu(hwnd), ID_MOD_REMOVE, MF_ENABLED);
			EnableMenuItem(GetMenu(hwnd), ID_MOD_ARCHIVE, MF_ENABLED);
		}
	}
	else
	{
		EnableWindow(GetDlgItem(hwnd, IDC_INFOBUTTON), FALSE);
		EnableWindow(GetDlgItem(hwnd, IDC_REMOVEBUTTON), FALSE);
		EnableWindow(GetDlgItem(hwnd, IDC_ARCHIVEBUTTON), FALSE);
		EnableMenuItem(GetMenu(hwnd), ID_MOD_INFO, MF_GRAYED);
		EnableMenuItem(GetMenu(hwnd), ID_MOD_REMOVE, MF_GRAYED);
		EnableMenuItem(GetMenu(hwnd), ID_MOD_ARCHIVE, MF_GRAYED);
	}
}

BOOL CALLBACK OverwriteDlgProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	switch(msg)
	{
	case WM_INITDIALOG:
		SetDlgItemText(hwnd, IDC_FILEINFOSTATIC, (LPCTSTR)lparam);
		return TRUE;

	case WM_COMMAND:
		switch(LOWORD(wparam))
		{
		case IDC_KEEPBUTTON:
			EndDialog(hwnd, 0);
			return TRUE;

		case IDC_REPLACEBUTTON:
			EndDialog(hwnd, 1);
			return TRUE;
		}
		break;

	case WM_DESTROY:
		return TRUE;
	}

	return FALSE;
}

BOOL CALLBACK AboutDlgProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	switch(msg)
	{
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		switch(LOWORD(wparam))
		{
		case IDOK:
			EndDialog(hwnd, 0);
			return TRUE;
		}
		break;

	case WM_CLOSE:
		EndDialog(hwnd, 0);
		return TRUE;

	case WM_DESTROY:
		return TRUE;
	}

	return FALSE;
}

BOOL CALLBACK InfoDlgProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	switch(msg)
	{
	case WM_INITDIALOG:
		{
			char buf[256];
			SetWindowText(hwnd, ModList.cMod->Name);
			for(int c = 0; c < ModList.cMod->nFiles; c++)
			{
				if(ModList.cMod->FileInfo[c].Type & MF_MANUAL)
				{
					sprintf(buf, "* %s", ModList.cMod->FileInfo[c].Name);
					SendDlgItemMessage(hwnd, IDC_FILELIST, LB_ADDSTRING, 0, (LPARAM)buf);
				}
				else
					SendDlgItemMessage(hwnd, IDC_FILELIST, LB_ADDSTRING, 0, (LPARAM)ModList.cMod->FileInfo[c].Name);
			}
		}
		return TRUE;

	case WM_COMMAND:
		switch(LOWORD(wparam))
		{
		case IDOK:
			EndDialog(hwnd, 0);
			return TRUE;
			
		case IDC_ADDBUTTON:
			if(AddOfn.Display(ghInstance, hwnd, "Add file...", OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_ALLOWMULTISELECT | OFN_EXPLORER, OFNWINDOW_OPEN))
			{
				int n = strlen(DFPath);
				char buf[256];
				if(strnicmp(AddOfn.GetFullPath(), DFPath, n) != 0)
				{
					MessageBox(hwnd, "File must be in Data Files folder.", "Error", MB_OK);
					return TRUE;
				}

				do{
					if(ModList.cMod->AddFile(&AddOfn.GetFullPath()[n], MF_MANUAL))
					{
						ModList.cMod->Size += file_exists(AddOfn.GetFullPath());
						sprintf(buf, "* %s", &AddOfn.GetFullPath()[n]);
						SendDlgItemMessage(hwnd, IDC_FILELIST, LB_ADDSTRING, 0, (LPARAM)buf);
					}
				}while(AddOfn.NextFile());
			}
			return TRUE;

		case IDC_REMOVEBUTTON:
			{
				char buf1[256], buf2[256];
				int n = SendDlgItemMessage(hwnd, IDC_FILELIST, LB_GETCURSEL, 0, 0);
				SendDlgItemMessage(hwnd, IDC_FILELIST, LB_GETTEXT, n, (LPARAM)buf1);
				SendDlgItemMessage(hwnd, IDC_FILELIST, LB_DELETESTRING, n, 0);
				sprintf(buf2, "%s%s", DFPath, &buf1[2]);
				if(ModList.cMod->RemoveFile(&buf1[2]))
					ModList.cMod->Size -= file_exists(buf2);
			}
			return TRUE;

		case IDC_FILELIST:
			if(HIWORD(wparam) == LBN_SELCHANGE)
			{
				char buf[256];
				int n = SendDlgItemMessage(hwnd, IDC_FILELIST, LB_GETCURSEL, 0, 0);
				if(n == LB_ERR)
					return TRUE;
				SendDlgItemMessage(hwnd, IDC_FILELIST, LB_GETTEXT, n, (LPARAM)buf);
				if(buf[0] == '*')
					EnableWindow(GetDlgItem(hwnd, IDC_REMOVEBUTTON), TRUE);
				else
					EnableWindow(GetDlgItem(hwnd, IDC_REMOVEBUTTON), FALSE);
			}
			return TRUE;
		}
		break;

	case WM_CLOSE:
		EndDialog(hwnd, 0);
		return TRUE;

	case WM_DESTROY:
		return TRUE;
	}

	return FALSE;
}

bool browse_folder(HWND hwnd, const char *title, char *path)
{
	LPITEMIDLIST lid = NULL;
	BROWSEINFO bi;
	IMalloc *im = NULL;
	char buf[MAX_PATH];

	bi.hwndOwner = hwnd;
	bi.pidlRoot = NULL;
	bi.pszDisplayName = buf;
	bi.lpszTitle = title;
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = NULL;
	bi.lParam = NULL;
	bi.iImage = 0;
	lid = SHBrowseForFolder(&bi);
	if(lid)
	{
		SHGetPathFromIDList(lid, path);
		SHGetMalloc(&im);
		im->Free(lid);
		im->Release();
		return true;
	}
	return false;
}

BOOL CALLBACK SettingsDlgProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	switch(msg)
	{
	case WM_INITDIALOG:
		SetDlgItemText(hwnd, IDC_MWPATHSTATIC, MWPath);
		SetDlgItemText(hwnd, IDC_ZIPSTATIC, ZipPath);
		if(hdr.remprompt)
			CheckDlgButton(hwnd, IDC_REMPROMPTCHECK, BST_CHECKED);
		if(hdr.dateformat == 0)
			CheckRadioButton(hwnd, IDC_MMDDYYYYRADIO, IDC_DDMMYYYYRADIO, IDC_MMDDYYYYRADIO);
		else
			CheckRadioButton(hwnd, IDC_MMDDYYYYRADIO, IDC_DDMMYYYYRADIO, IDC_DDMMYYYYRADIO);
		return TRUE;

	case WM_COMMAND:
		switch(LOWORD(wparam))
		{
		case IDC_PATHBUTTON:
			if(get_mw_browse_path(hwnd))
				SetDlgItemText(hwnd, IDC_MWPATHSTATIC, MWPath);
			return TRUE;

		case IDC_ZIPBUTTON:
			if(browse_folder(hwnd, "Select default zip file path", ZipPath))
			{
				check_path(ZipPath);
				SetDlgItemText(hwnd, IDC_ZIPSTATIC, ZipPath);
			}
			return TRUE;

		case IDOK:
			hdr.remprompt = IsDlgButtonChecked(hwnd, IDC_REMPROMPTCHECK) == BST_CHECKED ? TRUE : FALSE;
			hdr.dateformat = IsDlgButtonChecked(hwnd, IDC_MMDDYYYYRADIO) == BST_CHECKED ? 0 : 1;
			EndDialog(hwnd, 1);
			return TRUE;
		}
		break;

	case WM_DESTROY:
		return TRUE;
	}

	return FALSE;
}

static void check_master_files(void)
{
	char buf[256];
	int res = 1;
	MOD *mod;

	sprintf(buf, "%sMorrowind.esm", DFPath);
	if (file_exists(buf)) { // make sure it's here (it better be)
		if(ModList.SelectMod("Morrowind.esm"))
		{
			res = compare_file_date(buf, &ModList.cMod->ModifyTime);
			if(res != 0)
			{
				ModList.cMod->RemoveAuto();
				mod = scan_esm(buf);
				mod->AddFile("Textures\\_land_default.tga", 0);
				for(int c = 1; c <= 10; c++)
				{
					sprintf(buf, "Sound\\Vo\\Misc\\CharGen QA%d.mp3", c);
					mod->AddFile(buf, 0);
				}
				ModList.cMod->Merge(mod);
				delete mod;
			}
		}
		else if(res == 1)
		{
			mod = scan_esm(buf);
			mod->AddFile("Textures\\_land_default.tga", 0);
			for(int c = 1; c <= 10; c++)
			{
				sprintf(buf, "Sound\\Vo\\Misc\\CharGen QA%d.mp3", c);
				mod->AddFile(buf, 0);
			}
			ModList.ModInfo.NewPtr(mod);
		}
		res = 1;
	}
	
	sprintf(buf, "%sTribunal.esm", DFPath);
	if (file_exists(buf)) { // make sure it's here
		if(ModList.SelectMod("Tribunal.esm"))
		{
			res = compare_file_date(buf, &ModList.cMod->ModifyTime);
			if(res != 0)
			{
				ModList.cMod->RemoveAuto();
				mod = scan_esm(buf);
				ModList.cMod->Merge(mod);
				delete mod;
			}
		}
		else if(res == 1)
		{
			mod = scan_esm(buf);
			ModList.ModInfo.NewPtr(mod);
		}
		res = 1;
	}

	sprintf(buf, "%sBloodmoon.esm", DFPath);
	if (file_exists(buf)) { // make sure it's here
		if(ModList.SelectMod("Bloodmoon.esm"))
		{
			res = compare_file_date(buf, &ModList.cMod->ModifyTime);
			if(res != 0)
			{
				ModList.cMod->RemoveAuto();
				mod = scan_esm(buf);
				ModList.cMod->Merge(mod);
				delete mod;
			}
		}
		else if(res == 1)
		{
			mod = scan_esm(buf);
			ModList.ModInfo.NewPtr(mod);
		}
	}

}

static void check_plugin_files(void)
{
	char buf1[256], buf2[256];
	int res = 1;
	MOD *mod;
	WIN32_FIND_DATA wfd;
	HANDLE hnd;
	memset(&wfd, 0, sizeof(wfd));

	sprintf(buf1, "%s*.esm", DFPath);
	hnd = FindFirstFile(buf1, &wfd);
	if(hnd != INVALID_HANDLE_VALUE)
	{
		do{
			if(stricmp(wfd.cFileName, "Morrowind.esm") != 0 && stricmp(wfd.cFileName, "Tribunal.esm") != 0 && stricmp(wfd.cFileName, "Bloodmoon.esm") != 0)
			{
				sprintf(buf2, "%s%s", DFPath, wfd.cFileName);
				if(ModList.SelectMod(wfd.cFileName))
				{
					res = memcmp(&ModList.cMod->ModifyTime, &wfd.ftLastWriteTime, sizeof(FILETIME));
					if(res != 0)
					{
						ModList.cMod->RemoveAuto();
						mod = scan_esp(buf2);
						ModList.cMod->Merge(mod);
						memcpy(&ModList.cMod->ModifyTime, &wfd.ftLastWriteTime, sizeof(FILETIME));
						delete mod;
					}
				}
				else if(res != 0)
				{
					mod = scan_esp(buf2);
					ModList.ModInfo.NewPtr(mod);
					memcpy(&mod->ModifyTime, &wfd.ftLastWriteTime, sizeof(FILETIME));
				}
			}
		}while(FindNextFile(hnd, &wfd));
		FindClose(hnd);
	}

	sprintf(buf1, "%s*.esp", DFPath);
	hnd = FindFirstFile(buf1, &wfd);
	if(hnd != INVALID_HANDLE_VALUE)
	{
		do{
			res = 1;
			sprintf(buf2, "%s%s", DFPath, wfd.cFileName);
			if(ModList.SelectMod(wfd.cFileName))
			{
				res = memcmp(&ModList.cMod->ModifyTime, &wfd.ftLastWriteTime, sizeof(FILETIME));
				if(res != 0)
				{
					ModList.cMod->RemoveAuto();
					mod = scan_esp(buf2);
					ModList.cMod->Merge(mod);
					memcpy(&ModList.cMod->ModifyTime, &wfd.ftLastWriteTime, sizeof(FILETIME));
					delete mod;
				}
			}
			else if(res != 0)
			{
				mod = scan_esp(buf2);
				ModList.ModInfo.NewPtr(mod);
				memcpy(&mod->ModifyTime, &wfd.ftLastWriteTime, sizeof(FILETIME));
			}
		}while(FindNextFile(hnd, &wfd));
		FindClose(hnd);
	}

	ModList.ModInfo.GotoStart();
	do{
		sprintf(buf1, "%s%s", DFPath, (const char *)ModList.ModInfo.GetItem().Name);
		if(!file_exists(buf1))
		{
			ModList.SelectMod(ModList.ModInfo.GetItem().Name);
			ModList.RemoveMod();
		}
	}while(ModList.ModInfo.NextItem());
}

static void delete_plugin(void)
{
	char buf[256];
	if(!ModList.cMod)
		return;

	for(int c = 0; c < ModList.cMod->nFiles; c++)
	{
		if(!ModList.FileShared(ModList.cMod->FileInfo[c].Name))
		{
			sprintf(buf, "%s%s", DFPath, ModList.cMod->FileInfo[c].Name);
			DeleteFile(buf);
			remove_dir(buf);
		}
	}
	ModList.RemoveMod();
}

static int CALLBACK compare_proc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	int res = 0;

	switch(LOBYTE(lParamSort))
	{
	case 0:
		res = stricmp(((MOD *)lParam1)->Name, ((MOD *)lParam2)->Name);
		break;
	
	case 1:
		res = ((MOD *)lParam1)->Size - ((MOD *)lParam2)->Size;
		break;

	case 2:
		res = ((MOD *)lParam1)->nFiles - ((MOD *)lParam2)->nFiles;
		break;

	case 3:
		res = CompareFileTime(&((MOD *)lParam1)->ModifyTime, &((MOD *)lParam2)->ModifyTime);
		break;
	}

    if(HIBYTE(lParamSort) == 1)
		return -res;
	return res;
}

static void init_listview(HWND hwnd)
{
	int n = 0;
	LVCOLUMN lvc;
	LVITEM lvi;

	memset(&lvc, 0, sizeof(LVCOLUMN));
	lvc.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM;
	lvc.fmt = LVCFMT_LEFT;
	memset(&lvi, 0, sizeof(LVITEM));
	lvi.mask = LVIF_TEXT | LVIF_PARAM | LVIF_STATE;
	lvi.state = 0;
	lvi.stateMask = 0;

	lvc.iSubItem = 0;
	lvc.pszText = "Name";
	lvc.cx = 175;
	if(ListView_InsertColumn(hwnd, 0, &lvc) == -1)
		return;
	lvc.iSubItem = 1;
	lvc.pszText = "Size";
	lvc.cx = 75;
	lvc.fmt = LVCFMT_RIGHT;
	ListView_InsertColumn(hwnd, 1, &lvc);
	lvc.iSubItem = 2;
	lvc.cx = 50;
	lvc.pszText = "Files";
	ListView_InsertColumn(hwnd, 2, &lvc);
	lvc.iSubItem = 3;
	lvc.cx = 75;
	lvc.pszText = "Modified";
	ListView_InsertColumn(hwnd, 3, &lvc);

	ModList.ModInfo.GotoStart();
	do{
		lvi.iItem = n;
		lvi.iSubItem = 0;
		lvi.lParam = (LPARAM)&ModList.ModInfo.GetItem();
		lvi.pszText = LPSTR_TEXTCALLBACK;
		ListView_InsertItem(hwnd, &lvi);
		n++;
	}while(ModList.ModInfo.NextItem());

	ListView_SortItems(hwnd, compare_proc, MAKEWORD(hdr.sortby, hdr.sortorder));
}

static void reinit_listview(HWND hwnd)
{
	int n = 0;
	LVITEM lvi;
	memset(&lvi, 0, sizeof(LVITEM));
	lvi.mask = LVIF_TEXT | LVIF_PARAM | LVIF_STATE;
	lvi.state = 0;
	lvi.stateMask = 0;
	ListView_DeleteAllItems(hwnd);
	ModList.ModInfo.GotoStart();
	do{
		lvi.iItem = n;
		lvi.iSubItem = 0;
		lvi.lParam = (LPARAM)&ModList.ModInfo.GetItem();
		lvi.pszText = LPSTR_TEXTCALLBACK;
		ListView_InsertItem(hwnd, &lvi);
		n++;
	}while(ModList.ModInfo.NextItem());
	ListView_SortItems(hwnd, compare_proc, MAKEWORD(hdr.sortby, hdr.sortorder));
}

BOOL CALLBACK MainDlgProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	static char lvbuf[32];
	switch(msg)
	{
	case WM_INITDIALOG:
		if(load_stuff(GetDlgItem(hwnd, IDC_MODLIST)) < 0x00020000)
		{
			if(!get_mw_reg_path())
			{
				if(!get_mw_browse_path(hwnd))
					DestroyWindow(hwnd);
			}
		}
		sprintf(DFPath, "%sData Files\\", MWPath);
		InstallOfn.SetInitialPath(ZipPath);
		InstallOfn.SetFilter("Zip files (*.zip),*.zip");
		InstallOfn.SetDefaultExt("zip");
		ScanOfn.SetInitialPath(DFPath);
		ScanOfn.SetFilter("Plugin files (*.esp),*.esp,Master files (*.esm),*.esm");
		AddOfn.SetInitialPath(DFPath);
		AddOfn.SetFilter("All files (*.*)");
		check_master_files();
		check_plugin_files();
		save_stuff();
		init_listview(GetDlgItem(hwnd, IDC_MODLIST));
		return TRUE;

	case WM_COMMAND:
		switch(LOWORD(wparam))
		{
		case ID_MOD_INFO:
		case IDC_INFOBUTTON:
			{
				int n = ListView_GetSelectionMark(GetDlgItem(hwnd, IDC_MODLIST));
				DialogBoxParam(ghInstance, MAKEINTRESOURCE(IDD_INFODIALOG), hwnd, InfoDlgProc, NULL);
				ListView_RedrawItems(GetDlgItem(hwnd, IDC_MODLIST), n, n);
				UpdateWindow(hwnd);
			}
			return TRUE;

		case ID_MOD_INSTALL:
		case IDC_INSTALLBUTTON:
			if(InstallOfn.Display(ghInstance, hwnd, "Install Plugin...", OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_ALLOWMULTISELECT | OFN_EXPLORER, OFNWINDOW_OPEN))
			{
				bool redraw = false;
				do{
					if(install_mod(InstallOfn.GetFullPath()))
						redraw = true;
				}while(InstallOfn.NextFile());
                if(redraw)
				{
					check_plugin_files();
					reinit_listview(GetDlgItem(hwnd, IDC_MODLIST));
					update_stuff(hwnd);
					save_stuff();
				}
			}
			return TRUE;

		case ID_MOD_ARCHIVE:
		case IDC_ARCHIVEBUTTON:
			{
				int l;
				char buf[256];
				strcpy(buf, ModList.cMod->Name);
				ModList.SelectMod(buf);
				l = strlen(buf);
				buf[l - 4] = '\0';
				sprintf(InstallOfn.CurrentPath, "%s.zip", buf);
				if(InstallOfn.Display(ghInstance, hwnd, "Archive Plugin...", OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, OFNWINDOW_SAVE))
					archive_mod(InstallOfn.GetFullPath(), ModList.cMod);
			}
			return TRUE;

		case ID_MOD_REMOVE:
		case IDC_REMOVEBUTTON:
			if(!hdr.remprompt || MessageBox(hwnd, "Are you sure you want to remove the selected plugin?", "Remove", MB_YESNO) == IDYES)
			{
				int n = ListView_GetSelectionMark(GetDlgItem(hwnd, IDC_MODLIST));
				ListView_DeleteItem(GetDlgItem(hwnd, IDC_MODLIST), n);
				delete_plugin();
				update_stuff(hwnd);
				save_stuff();
			}
			return TRUE;

		case ID_HELP_ABOUT:
			DialogBoxParam(ghInstance, MAKEINTRESOURCE(IDD_ABOUTDIALOG), hwnd, AboutDlgProc, NULL);
			return TRUE;

		case ID_MOD_EXIT:
			PostMessage(hwnd, WM_CLOSE, 0, 0);
			return TRUE;

		case ID_OPTIONS_SETTINGS:
			DialogBoxParam(ghInstance, MAKEINTRESOURCE(IDD_SETTINGSDIALOG), hwnd, SettingsDlgProc, NULL);
			save_stuff();
			ListView_RedrawItems(GetDlgItem(hwnd, IDC_MODLIST), 0, ModList.ModInfo.Length() - 1);
			UpdateWindow(hwnd);
			return TRUE;

		/*case ID_MOD_SCANPLUGIN:
			if(ScanOfn.Display(ghInstance, hwnd, "Scan Plugin...", OFN_HIDEREADONLY | OFN_FILEMUSTEXIST, OFNWINDOW_OPEN))
			{
				MOD *mod;
				mod = scan_esp(ScanOfn.GetFullPath());
				if(mod)
				{
					SendDlgItemMessage(hwnd, IDC_MODLIST, LB_ADDSTRING, 0, (LPARAM)((const char *)mod->Name));
					update_stuff(hwnd);
					save_stuff();
				}
			}
			return TRUE;*/
		}
		break;

	case WM_NOTIFY:
		switch(((LPNMHDR)lparam)->code)
		{
		case LVN_GETDISPINFO:
			{
				LV_DISPINFO *lpvdi = (LV_DISPINFO *)lparam;
                FILETIME ft;
				SYSTEMTIME st;
				float size;
				switch(lpvdi->item.iSubItem)
				{
				case 0:
					lpvdi->item.pszText = ((MOD *)lpvdi->item.lParam)->Name;
					break;

				case 1:
					size = (float)((MOD *)lpvdi->item.lParam)->Size;
					if(size < 102400)
						sprintf(lvbuf, "%d Bytes", ((MOD *)lpvdi->item.lParam)->Size);
					else if(size < 1024 * 10240)
						sprintf(lvbuf, "%.2f KB", size / 1024.0f);
					else
						sprintf(lvbuf, "%.2f MB", size / (1024.0f * 1024.0f));
					lpvdi->item.pszText = lvbuf;
					break;

				case 2:
					itoa(((MOD *)lpvdi->item.lParam)->nFiles, lvbuf, 10);
					lpvdi->item.pszText = lvbuf;
					break;

				case 3:
					FileTimeToLocalFileTime(&((MOD *)lpvdi->item.lParam)->ModifyTime, &ft);
					FileTimeToSystemTime(&ft, &st);
					if(hdr.dateformat == 0)
						sprintf(lvbuf, "%d/%d/%d", st.wMonth, st.wDay, st.wYear);
					else
						sprintf(lvbuf, "%d/%d/%d", st.wDay, st.wMonth, st.wYear);
					lpvdi->item.pszText = lvbuf;
					break;
				}
				return TRUE;
			}
			break;

		case LVN_COLUMNCLICK:
			if(((LPNMLISTVIEW)lparam)->iSubItem == hdr.sortby)
			{
				hdr.sortorder++;
				if(hdr.sortorder > 1)
					hdr.sortorder = 0;
			}
			else
				hdr.sortby = ((LPNMLISTVIEW)lparam)->iSubItem;
			ListView_SortItems(GetDlgItem(hwnd, IDC_MODLIST), compare_proc, MAKEWORD(hdr.sortby, hdr.sortorder));
			return TRUE;

		case LVN_KEYDOWN:
			if(((LPNMLVKEYDOWN)lparam)->wVKey == VK_UP)
			{
				update_stuff(hwnd, -1);
				return TRUE;
			}
			if(((LPNMLVKEYDOWN)lparam)->wVKey == VK_DOWN)
			{
				update_stuff(hwnd, 1);
				return TRUE;
			}
			break;

		case NM_CLICK:
			update_stuff(hwnd);
			return TRUE;

		case NM_DBLCLK:
			{
				int n = ListView_GetSelectionMark(GetDlgItem(hwnd, IDC_MODLIST));
				DialogBoxParam(ghInstance, MAKEINTRESOURCE(IDD_INFODIALOG), hwnd, InfoDlgProc, NULL);
				ListView_RedrawItems(GetDlgItem(hwnd, IDC_MODLIST), n, n);
				UpdateWindow(hwnd);
			}
			return TRUE;
		}
		break;

	case WM_CLOSE:
		DestroyWindow(hwnd);
		return TRUE;

	case WM_DESTROY:
		PostQuitMessage(0);
		return TRUE;
	}

	return FALSE;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdList, int nCmdShow)
{
	MSG msg;
	HICON hicon;
	HANDLE hmutex;

	hmutex = CreateMutex(NULL, TRUE, "TESMU_Mutex");
	if(GetLastError() == ERROR_ALREADY_EXISTS)
		return 0;
	ghInstance = hInstance;
	InitCommonControls();
	GetCurrentDirectory(256, LocalPath);
	check_path(LocalPath);
	hMain = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_MAINDIALOG), NULL, MainDlgProc);
	hicon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_APPICON));
	SendMessage(hMain, WM_SETICON, ICON_BIG, (LPARAM)hicon);
	
	while(GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
    CloseHandle(hmutex);
	return msg.wParam;
}
